# Contributing to Lettuce

## Quickstart
 
 For the impatient, if you want to submit a quick pull request:

* Don't create a pull request upfront. Create a feature request ticket first so we can discuss your idea.
* Upon agreeing the feature is a good fit for Lettuce, please:
  * Make sure there is a ticket in GitHub issues.
  * Make sure you use the code formatters provided here and have them applied to your changes. Don’t submit any formatting related changes.
  * Make sure you submit test cases (unit or integration tests) that back your changes.
  * Try to reuse existing test sample code (domain classes). Try not to amend existing test cases but create new ones dedicated to the changes you’re making to the codebase. Try to test as locally as possible but potentially also add integration tests.

When submitting code, please make every effort to follow existing conventions and style in order to keep the code as readable as possible. Formatting changes create a lot of noise and reduce the likelyhood of merging the pull request.
Formatting settings are provided for Eclipse in https://github.com/lettuce-io/lettuce-core/blob/master/formatting.xml

## Bugreports

If you report a bug, please ensure to specify the following:

* Check GitHub issues whether someone else already filed a ticket. If not, then feel free to create another one.
* Comments on closed tickets are typically a bad idea as there's little attention on closed tickets. Please create a new one.
* Lettuce version (e.g. 3.2.Final).
* Contextual information (what were you trying to do using Lettuce).
* Simplest possible steps to reproduce:
   * Ideally, a [Minimal, Complete, and Verifiable example](https://stackoverflow.com/help/mcve).
   * JUnit tests to reproduce are great but not obligatory.

## Features

If you want to request a feature, please ensure to specify the following:

* What do you want to achieve?
* Contextual information (what were you trying to do using Lettuce).
* Ideally, but not required: Describe your idea how you would implement it.

## Questions

If you have a question, then check one of the following places first. Typically, forums, chats, and mailing lists are the best place to ask your question as you can expect to get an answer faster there:

**Checkout the docs**

* [Reference documentation](https://lettuce.io/docs/)
* [Wiki](https://github.com/lettuce-io/lettuce-core/wiki)
* [Javadoc](https://lettuce.io/core/release/api/)

**Communication**

* Google Group/Mailing List (General discussion, announcements and releases): [lettuce-redis-client-users](https://groups.google.com/d/forum/lettuce-redis-client-users) or [lettuce-redis-client-users@googlegroups.com](mailto:lettuce-redis-client-users@googlegroups.com)
* Stack Overflow (Questions): [Questions about Lettuce](https://stackoverflow.com/questions/tagged/lettuce)
* Gitter (General discussion): [![Join the chat at https://gitter.im/lettuce-io/Lobby](https://badges.gitter.im/Join%20Chat.svg)](https://gitter.im/lettuce-io/Lobby?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)

## License

By contributing your code, you agree to license your contribution under the terms of [Apache License 2.0] (http://www.apache.org/licenses/LICENSE-2.0).

All files are released with the Apache 2.0 license.
